import * as dntShim from "../_dnt.shims.js";
import { isAbsolute as isPathAbsolute } from "../deps/jsr.io/@std/path/1.0.7/is_absolute.js";
import { EOL } from "../deps/jsr.io/@std/fs/1.0.5/eol.js";
import { getEnv } from "@hugoalh/env/env";
import { isStringSingleLine } from "@hugoalh/is-string-singleline";
/**
 * GitHub Actions file command type.
 */
export var GitHubActionsFileCommandType;
(function (GitHubActionsFileCommandType) {
    GitHubActionsFileCommandType["pairs"] = "pairs";
    GitHubActionsFileCommandType["Pairs"] = "pairs";
    GitHubActionsFileCommandType["raw"] = "raw";
    GitHubActionsFileCommandType["Raw"] = "raw";
    GitHubActionsFileCommandType["values"] = "values";
    GitHubActionsFileCommandType["Values"] = "values";
})(GitHubActionsFileCommandType || (GitHubActionsFileCommandType = {}));
const commandsFile = [
    {
        name: "GITHUB_ENV",
        type: "pairs"
    },
    {
        name: "GITHUB_OUTPUT",
        type: "pairs"
    },
    {
        name: "GITHUB_PATH",
        type: "values"
    },
    {
        name: "GITHUB_STATE",
        type: "pairs"
    },
    {
        name: "GITHUB_STEP_SUMMARY",
        type: "raw"
    }
];
const regexpCommandFile = /^(?:[\dA-Z][\dA-Z_-]*)?[\dA-Z]$/;
/**
 * **\[🅰️ Advanced\]** Get the file command path in order to communicate with the GitHub Actions runner via the file command.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - *Resources*
 * >   - File System - Read (`read`)
 * >     - *Resources*
 * > - NodeJS (>= v20.9.0) 🧪
 * >   - File System - Read (`fs-read`)
 * >     - *Resources*
 * @param {string} command File command.
 * @returns {string} Path of the file command.
 */
export function getFileCommandPath(command) {
    if (!(commandsFile.map(({ name }) => {
        return name;
    }).includes(command) ||
        regexpCommandFile.test(command))) {
        throw new SyntaxError(`\`${command}\` is not a valid GitHub Actions file command!`);
    }
    const path = getEnv(command) ?? "";
    if (path.length === 0) {
        throw new Error(`File command \`${command}\` path is not defined!`);
    }
    if (!isPathAbsolute(path)) {
        throw new Error(`\`${path}\` (file command \`${command}\`) is not a valid absolute path!`);
    }
    try {
        const { isFile } = dntShim.Deno.statSync(path);
        if (!isFile) {
            throw new Error(`\`${path}\` (file command \`${command}\`) is not a file!`);
        }
    }
    catch (error) {
        // NOTE: Command file may not created yet.
        if (!(error instanceof dntShim.Deno.errors.NotFound)) {
            throw error;
        }
    }
    return path;
}
/**
 * Format GitHub Actions file pairs command.
 * @param {Map<string, string>} inputs Inputs.
 * @returns {string}
 */
function formatFilePairsCommand(inputs) {
    return Array.from(inputs.entries(), ([key, value]) => {
        if (isStringSingleLine(value)) {
            return `${key}=${value}`;
        }
        let delimiter;
        do {
            delimiter = crypto.randomUUID().replaceAll("-", "");
        } while (key.search(delimiter) !== -1 ||
            value.search(delimiter) !== -1);
        return `${key}<<${delimiter}${EOL}${value.replace(/\r?\n/g, EOL)}\n${delimiter}`;
    }).join(EOL);
}
/**
 * **\[🅰️ Advanced\]** Append value to the file line command.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - *Resources*
 * >   - File System - Read (`read`)
 * >     - *Resources*
 * >   - File System - Write (`write`)
 * >     - *Resources*
 * > - NodeJS (>= v20.9.0) 🧪
 * >   - File System - Read (`fs-read`)
 * >     - *Resources*
 * >   - File System - Write (`fs-write`)
 * >     - *Resources*
 * @param {string} command File command.
 * @param {...string} values Value of the file line command.
 * @returns {void}
 */
export function appendFileLineCommand(command, ...values) {
    const path = getFileCommandPath(command);
    values.forEach((value) => {
        if (!(isStringSingleLine(value) && value.length > 0)) {
            throw new SyntaxError(`\`${value}\` is not a valid GitHub Actions file line command value!`);
        }
    });
    if (values.length > 0) {
        dntShim.Deno.writeTextFileSync(path, `${Array.from(new Set(values).values()).join(EOL)}${EOL}`, { append: true });
    }
}
export function appendFileMapCommand(command, param1, param2) {
    const path = getFileCommandPath(command);
    const pairs = (typeof param1 === "string") ? new Map([[param1, param2]])
        : (param1 instanceof Map) ? param1
            : new Map(Object.entries(param1));
    pairs.forEach((_value, key) => {
        if (!isStringSingleLine(key)) {
            throw new SyntaxError(`\`${key}\` is not a valid GitHub Actions file map command pair key!`);
        }
    });
    if (pairs.size > 0) {
        dntShim.Deno.writeTextFileSync(path, `${formatFilePairsCommand(pairs)}${EOL}`, { append: true });
    }
}
/**
 * **\[🅰️ Advanced\]** Clear the file command which set in the current step.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - *Resources*
 * >   - File System - Read (`read`)
 * >     - *Resources*
 * >   - File System - Write (`write`)
 * >     - *Resources*
 * > - NodeJS (>= v20.9.0) 🧪
 * >   - File System - Read (`fs-read`)
 * >     - *Resources*
 * >   - File System - Write (`fs-write`)
 * >     - *Resources*
 * @param {string} command File command.
 * @returns {void}
 */
export function clearFileCommand(command) {
    return dntShim.Deno.writeTextFileSync(getFileCommandPath(command), "");
}
/**
 * **\[🅰️ Advanced\]** Optimize the file command to reduce size whenever possible.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - *Resources*
 * >   - File System - Read (`read`)
 * >     - *Resources*
 * >   - File System - Write (`write`)
 * >     - *Resources*
 * > - NodeJS (>= v20.9.0) 🧪
 * >   - File System - Read (`fs-read`)
 * >     - *Resources*
 * >   - File System - Write (`fs-write`)
 * >     - *Resources*
 * @param {string} command File command.
 * @param {GitHubActionsFileCommandType | keyof typeof GitHubActionsFileCommandType} [type="raw"] Type of the file command; Only used when the {@linkcode command} is not known.
 * @returns {void}
 */
export function optimizeFileCommand(command, type = "raw") {
    const path = getFileCommandPath(command);
    switch (commandsFile.find(({ name }) => {
        return (name === command);
    })?.type ?? GitHubActionsFileCommandType[type]) {
        case "pairs": {
            const pairs = new Map();
            const content = dntShim.Deno.readTextFileSync(path).split(/\r?\n/g);
            for (let index = 0; index < content.length; index += 1) {
                const line = content[index];
                if (/^[\s\t]*$/.test(line)) {
                    continue;
                }
                if (/^.+<<.+?$/.test(line)) {
                    const lineSplit = line.split("<<");
                    const key = lineSplit.slice(0, lineSplit.length - 1).join("<<");
                    const delimiter = lineSplit[-1];
                    const value = [];
                    let indexOffset = index;
                    while (true) {
                        indexOffset += 1;
                        if (indexOffset >= content.length) {
                            // NOTE: File may contain issues, abort optimization.
                            return;
                        }
                        const lineOffset = content[indexOffset];
                        if (lineOffset === delimiter) {
                            break;
                        }
                        value.push(lineOffset);
                    }
                    pairs.set(key, value.join("\n"));
                    index = indexOffset;
                    continue;
                }
                if (/^.+?=.+$/.test(line)) {
                    const [key, value] = line.split("=", 1);
                    pairs.set(key, value);
                    continue;
                }
                // NOTE: File may contain issues, abort optimization.
                return;
            }
            return dntShim.Deno.writeTextFileSync(path, (pairs.size > 0) ? `${formatFilePairsCommand(pairs)}${EOL}` : "");
        }
        case "raw":
            return;
        case "values": {
            const content = new Set(dntShim.Deno.readTextFileSync(path).split(/\r?\n/g).map((value) => {
                return value.trim();
            }).filter((value) => {
                return (value.length > 0);
            }));
            return dntShim.Deno.writeTextFileSync(path, (content.size > 0) ? `${Array.from(content.values()).join(EOL)}${EOL}` : "");
        }
        default:
            throw new RangeError(`\`${type}\` is not a valid GitHub Actions file command type! Only accept these values: ${Object.keys(GitHubActionsFileCommandType).join(", ")}`);
    }
}
;
