import { isAbsolute as isPathAbsolute } from "./deps/jsr.io/@std/path/1.0.7/is_absolute.js";
import { setEnv } from "@hugoalh/env/env";
import { addEnvPath } from "@hugoalh/env/path";
import { isStringSingleLine } from "@hugoalh/is-string-singleline";
import { appendFileLineCommand, appendFileMapCommand, clearFileCommand, optimizeFileCommand } from "./command/file.js";
export function addPATH(param0, options = {}) {
    const { scopeCurrent = true, scopeSubsequent = true } = options;
    const paths = (typeof param0 === "string") ? [param0] : param0;
    paths.forEach((path) => {
        if (!isPathAbsolute(path)) {
            throw new SyntaxError(`\`${path}\` is not a valid absolute path!`);
        }
    });
    if (paths.length > 0) {
        if (scopeCurrent) {
            addEnvPath(...paths);
        }
        if (scopeSubsequent) {
            appendFileLineCommand("GITHUB_PATH", ...paths);
        }
    }
}
/**
 * **\[🅰️ Advanced\]** Clear the environment variables for all of the subsequent steps which set in the current step.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_ENV`
 * >   - File System - Read (`read`)
 * >     - *Resources*
 * >   - File System - Write (`write`)
 * >     - *Resources*
 * > - NodeJS (>= v20.9.0) 🧪
 * >   - File System - Read (`fs-read`)
 * >     - *Resources*
 * >   - File System - Write (`fs-write`)
 * >     - *Resources*
 * @returns {void}
 */
export function clearEnvironmentVariableSubsequent() {
    clearFileCommand("GITHUB_ENV");
}
export { clearEnvironmentVariableSubsequent as clearEnvSubsequent };
/**
 * **\[🅰️ Advanced\]** Clear the `PATH` for all of the subsequent steps which set in the current step.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_PATH`
 * >   - File System - Read (`read`)
 * >     - *Resources*
 * >   - File System - Write (`write`)
 * >     - *Resources*
 * > - NodeJS (>= v20.9.0) 🧪
 * >   - File System - Read (`fs-read`)
 * >     - *Resources*
 * >   - File System - Write (`fs-write`)
 * >     - *Resources*
 * @returns {void}
 */
export function clearPATHSubsequent() {
    clearFileCommand("GITHUB_PATH");
}
/**
 * **\[🅰️ Advanced\]** Optimize the environment variables for all of the subsequent steps which set in the current step to reduce size whenever possible.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_ENV`
 * >   - File System - Read (`read`)
 * >     - *Resources*
 * >   - File System - Write (`write`)
 * >     - *Resources*
 * > - NodeJS (>= v20.9.0) 🧪
 * >   - File System - Read (`fs-read`)
 * >     - *Resources*
 * >   - File System - Write (`fs-write`)
 * >     - *Resources*
 * @returns {void}
 */
export function optimizeEnvironmentVariableSubsequent() {
    optimizeFileCommand("GITHUB_ENV");
}
export { optimizeEnvironmentVariableSubsequent as optimizeEnvSubsequent };
/**
 * **\[🅰️ Advanced\]** Optimize the `PATH` for all of the subsequent steps which set in the current step to reduce size whenever possible.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_PATH`
 * >   - File System - Read (`read`)
 * >     - *Resources*
 * >   - File System - Write (`write`)
 * >     - *Resources*
 * > - NodeJS (>= v20.9.0) 🧪
 * >   - File System - Read (`fs-read`)
 * >     - *Resources*
 * >   - File System - Write (`fs-write`)
 * >     - *Resources*
 * @returns {void}
 */
export function optimizePATHSubsequent() {
    optimizeFileCommand("GITHUB_PATH");
}
const regexpEnvironmentVariableKeyForbidden = /^(?:CI|PATH)$|^(?:ACTIONS|GITHUB|RUNNER)_/i;
/**
 * Validate the item is a valid GitHub Actions environment variable key.
 * @param {string} item Item that need to determine.
 * @returns {void}
 */
function validateEnvironmentVariableKey(item) {
    if (!(isStringSingleLine(item) && item.length > 0)) {
        throw new SyntaxError(`\`${item}\` is not a valid environment variable key!`);
    }
    if (regexpEnvironmentVariableKeyForbidden.test(item)) {
        throw new Error(`Modify environment variable \`${item}\` is forbidden!`);
    }
}
export function setEnvironmentVariable(param0, param1, param2) {
    const { scopeCurrent = true, scopeSubsequent = true } = ((typeof param0 === "string") ? param1 : param2) ?? {};
    const pairs = new Map();
    if (typeof param0 === "string") {
        validateEnvironmentVariableKey(param0);
        pairs.set(param0, param1);
    }
    else {
        for (const [key, value] of ((param0 instanceof Map) ? param0.entries() : Object.entries(param0))) {
            validateEnvironmentVariableKey(key);
            pairs.set(key, value);
        }
    }
    if (pairs.size > 0) {
        if (scopeCurrent) {
            for (const [key, value] of pairs.values()) {
                setEnv(key, value);
            }
        }
        if (scopeSubsequent) {
            appendFileMapCommand("GITHUB_ENV", pairs);
        }
    }
}
export { setEnvironmentVariable as setEnv };
