import { getEnv } from "@hugoalh/env/env";
import { isJSONPrimitive, } from "@hugoalh/is-json";
import { isStringSingleLine } from "@hugoalh/is-string-singleline";
import { appendFileMapCommand, clearFileCommand, optimizeFileCommand } from "./command/file.js";
/**
 * **\[🅰️ Advanced\]** Clear the outputs which set in the current step.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_OUTPUT`
 * >   - File System - Read (`read`)
 * >     - *Resources*
 * >   - File System - Write (`write`)
 * >     - *Resources*
 * > - NodeJS (>= v20.9.0) 🧪
 * >   - File System - Read (`fs-read`)
 * >     - *Resources*
 * >   - File System - Write (`fs-write`)
 * >     - *Resources*
 * @returns {void}
 */
export function clearOutput() {
    clearFileCommand("GITHUB_OUTPUT");
}
/**
 * **\[🅰️ Advanced\]** Clear the states which set in the current step.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_STATE`
 * >   - File System - Read (`read`)
 * >     - *Resources*
 * >   - File System - Write (`write`)
 * >     - *Resources*
 * > - NodeJS (>= v20.9.0) 🧪
 * >   - File System - Read (`fs-read`)
 * >     - *Resources*
 * >   - File System - Write (`fs-write`)
 * >     - *Resources*
 * @returns {void}
 */
export function clearState() {
    clearFileCommand("GITHUB_STATE");
}
const regexpBigInt = /^(?:0b1*[01]+|0o[1-7]*[0-7]+|[1-9]*\d|0x[1-9A-Fa-f]*[\dA-Fa-f]+)n?$/;
const regexpBooleanFalse = /^[Ff]alse$|^FALSE$/;
const regexpBooleanTrue = /^[Tt]rue$|^TRUE$/;
/**
 * Get the raw value of a parameter.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - *Resources*
 * @param {string} source Source of the parameter.
 * @param {string} key Key of the parameter.
 * @returns {string} Raw value of the parameter.
 */
function getParameter(source, key) {
    if (!isStringSingleLine(key)) {
        throw new SyntaxError(`\`${key}\` is not a valid GitHub Actions ${source.toLowerCase()} key!`);
    }
    return getEnv(`${source.toUpperCase()}_${key.replaceAll(" ", "_").toUpperCase()}`) ?? "";
}
export function getInput(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getParameter("input", key);
    if (value.length === 0) {
        if (require) {
            throw new ReferenceError(`Input \`${key}\` is not defined!`);
        }
        return (fallback ? "" : undefined);
    }
    return value;
}
export { getInput as getInputString };
export function getInputBigInt(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getParameter("input", key);
    if (value.length === 0) {
        if (require) {
            throw new ReferenceError(`Input \`${key}\` is not defined!`);
        }
        return (fallback ? 0n : undefined);
    }
    try {
        if (!regexpBigInt.test(value)) {
            throw undefined;
        }
        return BigInt(value.replace(/n$/, ""));
    }
    catch {
        throw new SyntaxError(`Input \`${key}\` is not a valid big integer!`);
    }
}
export { getInputBigInt as getInputBigInteger };
export function getInputBoolean(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getParameter("input", key);
    if (value.length === 0) {
        if (require) {
            throw new ReferenceError(`Input \`${key}\` is not defined!`);
        }
        return (fallback ? false : undefined);
    }
    if (regexpBooleanFalse.test(value)) {
        return false;
    }
    if (regexpBooleanTrue.test(value)) {
        return true;
    }
    throw new SyntaxError(`Input \`${key}\` is not a valid boolean!`);
}
export function getInputJSON(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getParameter("input", key);
    if (value.length === 0) {
        if (require) {
            throw new ReferenceError(`Input \`${key}\` is not defined!`);
        }
        return (fallback ? null : undefined);
    }
    try {
        return JSON.parse(value);
    }
    catch {
        throw new SyntaxError(`Input \`${key}\` is not a valid JSON value!`);
    }
}
export { getInputJSON as getInputJSONValue };
export function getInputJSONArray(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getInputJSON(key, {
        //@ts-ignore All of the overloads are fulfilled.
        require
    });
    if (value === null) {
        return (fallback ? [] : undefined);
    }
    if (!Array.isArray(value)) {
        throw new SyntaxError(`Input \`${key}\` is not a valid JSON array!`);
    }
    return value;
}
export function getInputJSONObject(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getInputJSON(key, {
        //@ts-ignore All of the overloads are fulfilled.
        require
    });
    if (value === null) {
        return (fallback ? {} : undefined);
    }
    if (!(typeof value === "object" && !Array.isArray(value))) {
        throw new SyntaxError(`Input \`${key}\` is not a valid JSON object!`);
    }
    return value;
}
export function getInputJSONPrimitive(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getInputJSON(key, {
        //@ts-ignore All of the overloads are fulfilled.
        require
    });
    if (value === null) {
        return (fallback ? null : undefined);
    }
    if (!isJSONPrimitive(value)) {
        throw new SyntaxError(`Input \`${key}\` is not a valid JSON primitive!`);
    }
    return value;
}
export function getInputNumber(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getParameter("input", key);
    if (value.length === 0) {
        if (require) {
            throw new ReferenceError(`Input \`${key}\` is not defined!`);
        }
        return (fallback ? 0 : undefined);
    }
    try {
        return Number(value);
    }
    catch {
        throw new SyntaxError(`Input \`${key}\` is not a valid number!`);
    }
}
export function getState(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getParameter("state", key);
    if (value.length === 0) {
        if (require) {
            throw new ReferenceError(`State \`${key}\` is not defined!`);
        }
        return (fallback ? "" : undefined);
    }
    return value;
}
export { getState as getStateString };
export function getStateBigInt(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getParameter("state", key);
    if (value.length === 0) {
        if (require) {
            throw new ReferenceError(`State \`${key}\` is not defined!`);
        }
        return (fallback ? 0n : undefined);
    }
    try {
        if (!regexpBigInt.test(value)) {
            throw undefined;
        }
        return BigInt(value.replace(/n$/, ""));
    }
    catch {
        throw new SyntaxError(`State \`${key}\` is not a valid big integer!`);
    }
}
export { getStateBigInt as getStateBigInteger };
export function getStateBoolean(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getParameter("state", key);
    if (value.length === 0) {
        if (require) {
            throw new ReferenceError(`State \`${key}\` is not defined!`);
        }
        return (fallback ? false : undefined);
    }
    if (regexpBooleanFalse.test(value)) {
        return false;
    }
    if (regexpBooleanTrue.test(value)) {
        return true;
    }
    throw new SyntaxError(`State \`${key}\` is not a valid boolean!`);
}
export function getStateJSON(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getParameter("state", key);
    if (value.length === 0) {
        if (require) {
            throw new ReferenceError(`State \`${key}\` is not defined!`);
        }
        return (fallback ? null : undefined);
    }
    try {
        return JSON.parse(value);
    }
    catch {
        throw new SyntaxError(`State \`${key}\` is not a valid JSON value!`);
    }
}
export { getStateJSON as getStateJSONValue };
export function getStateJSONArray(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getStateJSON(key, {
        //@ts-ignore All of the overloads are fulfilled.
        require
    });
    if (value === null) {
        return (fallback ? [] : undefined);
    }
    if (!Array.isArray(value)) {
        throw new SyntaxError(`State \`${key}\` is not a valid JSON array!`);
    }
    return value;
}
export function getStateJSONObject(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getStateJSON(key, {
        //@ts-ignore All of the overloads are fulfilled.
        require
    });
    if (value === null) {
        return (fallback ? {} : undefined);
    }
    if (!(typeof value === "object" && !Array.isArray(value))) {
        throw new SyntaxError(`State \`${key}\` is not a valid JSON object!`);
    }
    return value;
}
export function getStateJSONPrimitive(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getStateJSON(key, {
        //@ts-ignore All of the overloads are fulfilled.
        require
    });
    if (value === null) {
        return (fallback ? null : undefined);
    }
    if (!isJSONPrimitive(value)) {
        throw new SyntaxError(`State \`${key}\` is not a valid JSON primitive!`);
    }
    return value;
}
export function getStateNumber(key, options = {}) {
    const { fallback = true, require = false } = options;
    const value = getParameter("state", key);
    if (value.length === 0) {
        if (require) {
            throw new ReferenceError(`State \`${key}\` is not defined!`);
        }
        return (fallback ? 0 : undefined);
    }
    try {
        return Number(value);
    }
    catch {
        throw new SyntaxError(`State \`${key}\` is not a valid number!`);
    }
}
/**
 * **\[🅰️ Advanced\]** Optimize the outputs which set in the current step to reduce size whenever possible.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_OUTPUT`
 * >   - File System - Read (`read`)
 * >     - *Resources*
 * >   - File System - Write (`write`)
 * >     - *Resources*
 * > - NodeJS (>= v20.9.0) 🧪
 * >   - File System - Read (`fs-read`)
 * >     - *Resources*
 * >   - File System - Write (`fs-write`)
 * >     - *Resources*
 * @returns {void}
 */
export function optimizeOutput() {
    optimizeFileCommand("GITHUB_OUTPUT");
}
/**
 * **\[🅰️ Advanced\]** Optimize the states which set in the current step to reduce size whenever possible.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_STATE`
 * >   - File System - Read (`read`)
 * >     - *Resources*
 * >   - File System - Write (`write`)
 * >     - *Resources*
 * > - NodeJS (>= v20.9.0) 🧪
 * >   - File System - Read (`fs-read`)
 * >     - *Resources*
 * >   - File System - Write (`fs-write`)
 * >     - *Resources*
 * @returns {void}
 */
export function optimizeState() {
    optimizeFileCommand("GITHUB_STATE");
}
export function setOutput(param0, param1) {
    const pairs = new Map();
    if (typeof param0 === "string") {
        if (!isStringSingleLine(param0)) {
            throw new SyntaxError(`\`${param0}\` is not a valid GitHub Actions output key!`);
        }
        pairs.set(param0, param1);
    }
    else {
        for (const [key, value] of ((param0 instanceof Map) ? param0.entries() : Object.entries(param0))) {
            if (!isStringSingleLine(key)) {
                throw new SyntaxError(`\`${key}\` is not a valid GitHub Actions output key!`);
            }
            pairs.set(key, value);
        }
    }
    if (pairs.size > 0) {
        appendFileMapCommand("GITHUB_OUTPUT", pairs);
    }
}
export function setState(param0, param1) {
    const pairs = new Map();
    if (typeof param0 === "string") {
        if (!isStringSingleLine(param0)) {
            throw new SyntaxError(`\`${param0}\` is not a valid GitHub Actions state key!`);
        }
        pairs.set(param0, param1);
    }
    else {
        for (const [key, value] of ((param0 instanceof Map) ? param0.entries() : Object.entries(param0))) {
            if (!isStringSingleLine(key)) {
                throw new SyntaxError(`\`${key}\` is not a valid GitHub Actions state key!`);
            }
            pairs.set(key, value);
        }
    }
    if (pairs.size > 0) {
        appendFileMapCommand("GITHUB_STATE", pairs);
    }
}
