export declare const runnerArchitectures: readonly ["ARM", "ARM64", "X64", "X86"];
/**
 * GitHub Actions runner architecture.
 */
export type GitHubActionsRunnerArchitecture = typeof runnerArchitectures[number];
/**
 * Get the architecture of the GitHub Actions runner.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `RUNNER_ARCH`
 * @returns {GitHubActionsRunnerArchitecture} Architecture of the GitHub Actions runner.
 * @example
 * ```ts
 * getRunnerArchitecture();
 * //=> "X64"
 * ```
 */
export declare function getRunnerArchitecture(): GitHubActionsRunnerArchitecture;
export { getRunnerArchitecture as getRunnerArch };
/**
 * Get the debug status of the GitHub Actions runner.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `RUNNER_DEBUG`
 * @returns {boolean} Debug status of the GitHub Actions runner.
 */
export declare function getRunnerDebugStatus(): boolean;
export { getRunnerDebugStatus as isRunnerDebug };
/**
 * Get the name of the GitHub Actions runner.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `RUNNER_NAME`
 * @returns {string} Name of the GitHub Actions runner.
 * @example
 * ```ts
 * getRunnerName();
 * //=> "Hosted Agent"
 * ```
 */
export declare function getRunnerName(): string;
export declare const runnerOSes: readonly ["Linux", "macOS", "Windows"];
/**
 * GitHub Actions runner OS.
 */
export type GitHubActionsRunnerOS = typeof runnerOSes[number];
/**
 * Get the OS of the GitHub Actions runner.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `RUNNER_OS`
 * @returns {GitHubActionsRunnerOS} OS of the GitHub Actions runner.
 * @example
 * ```ts
 * getRunnerOS();
 * //=> "Windows"
 * ```
 */
export declare function getRunnerOS(): GitHubActionsRunnerOS;
/**
 * Get the path of the `TEMP` of the GitHub Actions runner.
 *
 * This directory is emptied at the beginning and end of each job, files will not be removed if the runner's user account does not have permission to delete them.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `RUNNER_TEMP`
 * @returns {string} Path of the `TEMP` of the GitHub Actions runner.
 * @example
 * ```ts
 * getRunnerTempPath();
 * //=> "D:\a\_temp"
 * ```
 */
export declare function getRunnerTempPath(): string;
/**
 * Get the path of the tool cache of the GitHub hosted GitHub Actions runner.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `RUNNER_TOOL_CACHE`
 * @returns {string | undefined} Path of the tool cache of the GitHub hosted GitHub Actions runner.
 * @example
 * ```ts
 * getRunnerToolCachePath();
 * //=> "C:\\hostedtoolcache\\windows"
 * ```
 */
export declare function getRunnerToolCachePath(): string | undefined;
/**
 * Get the path of the workspace of the GitHub Actions runner; The default working directory on the runner for steps.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_WORKSPACE`
 * @returns {string} Path of the workspace of the GitHub Actions runner.
 * @example
 * ```ts
 * getRunnerWorkspacePath();
 * //=> "/home/runner/work/my-repo-name/my-repo-name"
 * ```
 */
export declare function getRunnerWorkspacePath(): string;
export interface GitHubActionsRunnerTestOptions {
    /**
     * Also test whether have artifact resources.
     * @default false
     */
    artifact?: boolean;
    /**
     * Also test whether have cache resources.
     * @default false
     */
    cache?: boolean;
    /**
     * Also test whether have OpenID Connect (OIDC) resources.
     * @default false
     */
    oidc?: boolean;
    /**
     * Also test whether have tool cache resources.
     * @default false
     */
    toolCache?: boolean;
}
/**
 * Test the current process whether is executing inside the GitHub Actions runner.
 *
 * If this test is mandatory, use function {@linkcode validateInRunner} instead.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `ACTIONS_CACHE_URL`
 * >     - `ACTIONS_ID_TOKEN_REQUEST_TOKEN`
 * >     - `ACTIONS_ID_TOKEN_REQUEST_URL`
 * >     - `ACTIONS_RESULTS_URL`
 * >     - `ACTIONS_RUNTIME_TOKEN`
 * >     - `ACTIONS_RUNTIME_URL`
 * >     - `CI`
 * >     - `GITHUB_ACTION`
 * >     - `GITHUB_ACTIONS`
 * >     - `GITHUB_ACTOR`
 * >     - `GITHUB_ACTOR_ID`
 * >     - `GITHUB_API_URL`
 * >     - `GITHUB_ENV`
 * >     - `GITHUB_EVENT_NAME`
 * >     - `GITHUB_EVENT_PATH`
 * >     - `GITHUB_GRAPHQL_URL`
 * >     - `GITHUB_JOB`
 * >     - `GITHUB_OUTPUT`
 * >     - `GITHUB_PATH`
 * >     - `GITHUB_REF_NAME`
 * >     - `GITHUB_REF_TYPE`
 * >     - `GITHUB_REPOSITORY`
 * >     - `GITHUB_REPOSITORY_ID`
 * >     - `GITHUB_REPOSITORY_OWNER`
 * >     - `GITHUB_REPOSITORY_OWNER_ID`
 * >     - `GITHUB_RETENTION_DAYS`
 * >     - `GITHUB_RUN_ATTEMPT`
 * >     - `GITHUB_RUN_ID`
 * >     - `GITHUB_RUN_NUMBER`
 * >     - `GITHUB_SERVER_URL`
 * >     - `GITHUB_SHA`
 * >     - `GITHUB_STATE`
 * >     - `GITHUB_STEP_SUMMARY`
 * >     - `GITHUB_WORKFLOW`
 * >     - `GITHUB_WORKFLOW_REF`
 * >     - `GITHUB_WORKFLOW_SHA`
 * >     - `GITHUB_WORKSPACE`
 * >     - `RUNNER_ARCH`
 * >     - `RUNNER_NAME`
 * >     - `RUNNER_OS`
 * >     - `RUNNER_TEMP`
 * >     - `RUNNER_TOOL_CACHE`
 * @param {GitHubActionsRunnerTestOptions} [options={}] Options.
 * @returns {boolean} Test result.
 */
export declare function isInRunner(options?: GitHubActionsRunnerTestOptions): boolean;
/**
 * Validate the current process whether is executing inside the GitHub Actions runner.
 *
 * If this test is optional, use function {@linkcode isInRunner} instead.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `ACTIONS_CACHE_URL`
 * >     - `ACTIONS_ID_TOKEN_REQUEST_TOKEN`
 * >     - `ACTIONS_ID_TOKEN_REQUEST_URL`
 * >     - `ACTIONS_RESULTS_URL`
 * >     - `ACTIONS_RUNTIME_TOKEN`
 * >     - `ACTIONS_RUNTIME_URL`
 * >     - `CI`
 * >     - `GITHUB_ACTION`
 * >     - `GITHUB_ACTIONS`
 * >     - `GITHUB_ACTOR`
 * >     - `GITHUB_ACTOR_ID`
 * >     - `GITHUB_API_URL`
 * >     - `GITHUB_ENV`
 * >     - `GITHUB_EVENT_NAME`
 * >     - `GITHUB_EVENT_PATH`
 * >     - `GITHUB_GRAPHQL_URL`
 * >     - `GITHUB_JOB`
 * >     - `GITHUB_OUTPUT`
 * >     - `GITHUB_PATH`
 * >     - `GITHUB_REF_NAME`
 * >     - `GITHUB_REF_TYPE`
 * >     - `GITHUB_REPOSITORY`
 * >     - `GITHUB_REPOSITORY_ID`
 * >     - `GITHUB_REPOSITORY_OWNER`
 * >     - `GITHUB_REPOSITORY_OWNER_ID`
 * >     - `GITHUB_RETENTION_DAYS`
 * >     - `GITHUB_RUN_ATTEMPT`
 * >     - `GITHUB_RUN_ID`
 * >     - `GITHUB_RUN_NUMBER`
 * >     - `GITHUB_SERVER_URL`
 * >     - `GITHUB_SHA`
 * >     - `GITHUB_STATE`
 * >     - `GITHUB_STEP_SUMMARY`
 * >     - `GITHUB_WORKFLOW`
 * >     - `GITHUB_WORKFLOW_REF`
 * >     - `GITHUB_WORKFLOW_SHA`
 * >     - `GITHUB_WORKSPACE`
 * >     - `RUNNER_ARCH`
 * >     - `RUNNER_NAME`
 * >     - `RUNNER_OS`
 * >     - `RUNNER_TEMP`
 * >     - `RUNNER_TOOL_CACHE`
 * @param {GitHubActionsRunnerTestOptions} [options={}] Options.
 * @returns {void}
 */
export declare function validateInRunner(options?: GitHubActionsRunnerTestOptions): void;
//# sourceMappingURL=runner.d.ts.map