import * as dntShim from "./_dnt.shims.js";
import { isAbsolute as isPathAbsolute } from "./deps/jsr.io/@std/path/1.0.7/is_absolute.js";
import { getEnv } from "@hugoalh/env/env";
/**
 * Get the URL of the GitHub API.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_API_URL`
 * @returns {URL} URL of the GitHub API.
 * @example
 * ```ts
 * getGitHubAPIURL();
 * //=> https://api.github.com
 * ```
 */
export function getGitHubAPIURL() {
    return new URL(getEnv("GITHUB_API_URL") ?? "https://api.github.com");
}
/**
 * Get the URL of the GitHub GraphQL API.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_GRAPHQL_URL`
 * @returns {URL} URL of the GitHub GraphQL API.
 * @example
 * ```ts
 * getGitHubGraphQLAPIURL();
 * //=> https://api.github.com/graphql
 * ```
 */
export function getGitHubGraphQLAPIURL() {
    return new URL(getEnv("GITHUB_GRAPHQL_URL") ?? "https://api.github.com/graphql");
}
/**
 * Get the URL of the GitHub server.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_SERVER_URL`
 * @returns {URL} URL of the GitHub server.
 * @example
 * ```ts
 * getGitHubServerURL();
 * //=> https://github.com
 * ```
 */
export function getGitHubServerURL() {
    return new URL(getEnv("GITHUB_SERVER_URL") ?? "https://github.com");
}
/**
 * Get the name of the workflow.
 *
 * If the workflow file does not specify a name, then the value is the full path of the workflow file in the repository.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_WORKFLOW`
 * @returns {string} Name of the workflow.
 * @example
 * ```ts
 * getWorkflowName();
 * //=> "My test workflow"
 * ```
 */
export function getWorkflowName() {
    const value = getEnv("GITHUB_WORKFLOW");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow name, environment variable \`GITHUB_WORKFLOW\` is not defined!`);
    }
    return value;
}
/**
 * Get the reference path of the workflow.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_WORKFLOW_REF`
 * @returns {string} Reference path of the workflow.
 * @example
 * ```ts
 * getWorkflowPath();
 * //=> "octocat/hello-world/.github/workflows/my-workflow.yml@refs/heads/my_branch"
 * ```
 */
export function getWorkflowReferencePath() {
    const value = getEnv("GITHUB_WORKFLOW_REF");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow reference path, environment variable \`GITHUB_WORKFLOW_REF\` is not defined!`);
    }
    return value;
}
/**
 * Get the repository of the workflow.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_REPOSITORY`
 * @returns {string} Repository of the workflow.
 * @example
 * ```ts
 * getWorkflowRepository();
 * //=> "octocat/Hello-World"
 * ```
 */
export function getWorkflowRepository() {
    const value = getEnv("GITHUB_REPOSITORY");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow repository, environment variable \`GITHUB_REPOSITORY\` is not defined!`);
    }
    return value;
}
/**
 * Get the repository ID of the workflow.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_REPOSITORY_ID`
 * @returns {string} Repository ID of the workflow.
 * @example
 * ```ts
 * getWorkflowRepositoryID();
 * //=> "123456789"
 * ```
 */
export function getWorkflowRepositoryID() {
    const value = getEnv("GITHUB_REPOSITORY_ID");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow repository ID, environment variable \`GITHUB_REPOSITORY_ID\` is not defined!`);
    }
    return value;
}
/**
 * Get the repository owner of the workflow.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_REPOSITORY_OWNER`
 * @returns {string} Repository owner of the workflow.
 * @example
 * ```ts
 * getWorkflowRepositoryOwner();
 * //=> "octocat"
 * ```
 */
export function getWorkflowRepositoryOwner() {
    const value = getEnv("GITHUB_REPOSITORY_OWNER");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow repository owner, environment variable \`GITHUB_REPOSITORY_OWNER\` is not defined!`);
    }
    return value;
}
/**
 * Get the repository owner ID of the workflow.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_REPOSITORY_OWNER_ID`
 * @returns {string} Repository owner ID of the workflow.
 * @example
 * ```ts
 * getWorkflowRepositoryOwnerID();
 * //=> "1234567"
 * ```
 */
export function getWorkflowRepositoryOwnerID() {
    const value = getEnv("GITHUB_REPOSITORY_OWNER_ID");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow repository owner ID, environment variable \`GITHUB_REPOSITORY_OWNER_ID\` is not defined!`);
    }
    return value;
}
/**
 * Get the action ID of the workflow run.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_ACTION`
 * @returns {string} Action ID of the workflow run.
 * @example
 * ```ts
 * getWorkflowRunActionID();
 * //=> "__repo-owner_name-of-action-repo"
 * ```
 */
export function getWorkflowRunActionID() {
    const value = getEnv("GITHUB_ACTION");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run action ID, environment variable \`GITHUB_ACTION\` is not defined!`);
    }
    return value;
}
/**
 * Get the actor ID of the workflow run.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_ACTOR_ID`
 * @returns {string} Actor ID of the workflow run.
 * @example
 * ```ts
 * getWorkflowRunActorID();
 * //=> "1234567"
 * ```
 */
export function getWorkflowRunActorID() {
    const value = getEnv("GITHUB_ACTOR_ID");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run actor ID, environment variable \`GITHUB_ACTOR_ID\` is not defined!`);
    }
    return value;
}
/**
 * Get the actor name that initiate the workflow run.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_ACTOR`
 * @returns {string} Actor name that initiate the workflow run.
 * @example
 * ```ts
 * getWorkflowRunActorName();
 * //=> "octocat"
 * ```
 */
export function getWorkflowRunActorName() {
    const value = getEnv("GITHUB_ACTOR");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run actor name, environment variable \`GITHUB_ACTOR\` is not defined!`);
    }
    return value;
}
/**
 * Get the commit SHA that trigger the workflow run.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_SHA`
 * @returns {string} Commit SHA that trigger the workflow run.
 * @example
 * ```ts
 * getWorkflowRunCommitSHA();
 * //=> "ffac537e6cbbf934b08745a378932722df287a53"
 * ```
 */
export function getWorkflowRunCommitSHA() {
    const value = getEnv("GITHUB_SHA");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run commit SHA, environment variable \`GITHUB_SHA\` is not defined!`);
    }
    return value;
}
export const eventsName = [
    "branch_protection_rule",
    "check_run",
    "check_suite",
    "create",
    "delete",
    "deployment",
    "deployment_status",
    "discussion",
    "discussion_comment",
    "fork",
    "gollum",
    "issue_comment",
    "issues",
    "label",
    "merge_group",
    "milestone",
    "page_build",
    "project", // Legacy.
    "project_card", // Legacy.
    "project_column", // Legacy.
    "public",
    "pull_request",
    "pull_request_comment",
    "pull_request_review",
    "pull_request_review_comment",
    "pull_request_target",
    "push",
    "registry_package",
    "release",
    "repository_dispatch",
    "schedule",
    "status",
    "watch",
    "workflow_call",
    "workflow_dispatch",
    "workflow_run"
];
/**
 * Get the event name of the workflow run.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_EVENT_NAME`
 * @returns {GitHubActionsEventName} Event name of the workflow run.
 * @example
 * ```ts
 * getWorkflowRunEventName();
 * //=> "workflow_dispatch"
 * ```
 */
export function getWorkflowRunEventName() {
    const value = getEnv("GITHUB_EVENT_NAME");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run event name, environment variable \`GITHUB_EVENT_NAME\` is not defined!`);
    }
    if (!eventsName.includes(value)) {
        throw new Error(`\`${value}\` is not a known GitHub Actions workflow run event name!`);
    }
    return value;
}
/**
 * Get the ID of the workflow run.
 *
 * This is unique for each workflow run within a repository, and does not change when re-run the workflow run.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_RUN_ID`
 * @returns {string} ID of the workflow run.
 * @example
 * ```ts
 * getWorkflowRunID();
 * //=> 1658821493
 * ```
 */
export function getWorkflowRunID() {
    const value = getEnv("GITHUB_RUN_ID");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run ID, environment variable \`GITHUB_RUN_ID\` is not defined!`);
    }
    return value;
}
/**
 * Get the job ID of the workflow run.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_JOB`
 * @returns {string} Job ID of the workflow run.
 * @example
 * ```ts
 * getWorkflowRunJobID();
 * //=> "greeting_job"
 * ```
 */
export function getWorkflowRunJobID() {
    const value = getEnv("GITHUB_JOB");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run job ID, environment variable \`GITHUB_JOB\` is not defined!`);
    }
    return value;
}
/**
 * Get the run number of the workflow.
 *
 * This is unique for each run of a particular workflow in a repository, begins at `1` for the workflow's first run, and increments with each new run; This number does not change when re-run the workflow run.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_RUN_NUMBER`
 * @returns {number} Run number of the workflow.
 * @example
 * ```ts
 * getWorkflowRunNumber();
 * //=> 3
 * ```
 */
export function getWorkflowRunNumber() {
    const value = getEnv("GITHUB_RUN_NUMBER");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run number, environment variable \`GITHUB_RUN_NUMBER\` is not defined!`);
    }
    return Number.parseInt(value, 10);
}
export const referenceTypes = [
    "branch",
    "tag"
];
/**
 * Get the reference of the workflow run.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_BASE_REF`
 * >     - `GITHUB_HEAD_REF`
 * >     - `GITHUB_REF`
 * >     - `GITHUB_REF_NAME`
 * >     - `GITHUB_REF_PROTECTED`
 * >     - `GITHUB_REF_TYPE`
 * @returns {GitHubReferenceMeta} Reference of the workflow run.
 */
export function getWorkflowRunReference() {
    const base = getEnv("GITHUB_BASE_REF");
    const full = getEnv("GITHUB_REF");
    if (typeof full === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run reference, environment variable \`GITHUB_REF\` is not defined!`);
    }
    const head = getEnv("GITHUB_HEAD_REF");
    const short = getEnv("GITHUB_REF_NAME");
    if (typeof short === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run reference, environment variable \`GITHUB_REF_NAME\` is not defined!`);
    }
    const type = getEnv("GITHUB_REF_TYPE");
    if (typeof type === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run reference, environment variable \`GITHUB_REF_TYPE\` is not defined!`);
    }
    if (!referenceTypes.includes(type)) {
        throw new Error(`\`${type}\` is not a known GitHub Actions workflow run reference type!`);
    }
    return {
        base: (typeof base === "string" && base.length > 0) ? base : undefined,
        full,
        head: (typeof head === "string" && head.length > 0) ? head : undefined,
        protected: getEnv("GITHUB_REF_PROTECTED") === "true",
        short,
        type: type
    };
}
/**
 * Get the retention days of the workflow run.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_RETENTION_DAYS`
 * @returns {number} Retention days of the workflow run.
 * @example
 * ```ts
 * getWorkflowRunRetentionDays();
 * //=> 90
 * ```
 */
export function getWorkflowRunRetentionDays() {
    const value = getEnv("GITHUB_RETENTION_DAYS");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run retention days, environment variable \`GITHUB_RETENTION_DAYS\` is not defined!`);
    }
    return Number.parseInt(value, 10);
}
/**
 * Get the run attempt of the workflow run.
 *
 * This is unique for each attempt of a particular workflow run in a repository, begins at `1` for the workflow run's first attempt, and increments with each re-run.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_RUN_ATTEMPT`
 * @returns {number} Run attempt of the workflow run.
 * @example
 * ```ts
 * getWorkflowRunRunAttempt();
 * //=> 3
 * ```
 */
export function getWorkflowRunRunAttempt() {
    const value = getEnv("GITHUB_RUN_ATTEMPT");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run run attempt, environment variable \`GITHUB_RUN_ATTEMPT\` is not defined!`);
    }
    return Number.parseInt(value, 10);
}
/**
 * Get the URL of the workflow run.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_REPOSITORY`
 * @returns {URL} URL of the workflow run.
 * @example
 * ```ts
 * getWorkflowRunURL();
 * //=> https://github.com/octocat/Hello-World/actions/runs/1658821493
 * ```
 */
export function getWorkflowRunURL() {
    const repository = getEnv("GITHUB_REPOSITORY");
    if (typeof repository === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run URI, environment variable \`GITHUB_REPOSITORY\` is not defined!`);
    }
    const serverURLString = getGitHubServerURL().toString();
    return new URL(`${serverURLString}${serverURLString.endsWith("/") ? "" : "/"}${repository}/actions/runs/${getWorkflowRunID()}`);
}
/**
 * Get the webhook event payload of the workflow run.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_EVENT_PATH`
 * >   - File System - Read (`read`)
 * >     - *Resources*
 * > - NodeJS (>= v20.9.0) 🧪
 * >   - File System - Read (`fs-read`)
 * >     - *Resources*
 * @returns {GitHubActionsWebhookEventPayload} Webhook event payload of the workflow run.
 */
export function getWorkflowRunWebhookEventPayload() {
    const path = getEnv("GITHUB_EVENT_PATH");
    if (typeof path === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run webhook event payload, environment variable \`GITHUB_EVENT_PATH\` is not defined!`);
    }
    if (!isPathAbsolute(path)) {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow run webhook event payload, \`${path}\` (environment variable \`GITHUB_EVENT_PATH\`) is not a valid absolute path!`);
    }
    return JSON.parse(dntShim.Deno.readTextFileSync(path));
}
/**
 * Get the SHA of the workflow.
 *
 * > **🛡️ Require Runtime Permissions**
 * >
 * > - Deno
 * >   - Environment Variable (`env`)
 * >     - `GITHUB_WORKFLOW_SHA`
 * @returns {string} SHA of the workflow.
 */
export function getWorkflowSHA() {
    const value = getEnv("GITHUB_WORKFLOW_SHA");
    if (typeof value === "undefined") {
        throw new ReferenceError(`Unable to get the GitHub Actions workflow SHA, environment variable \`GITHUB_WORKFLOW_SHA\` is not defined!`);
    }
    return value;
}
